import java.io.File;
import java.io.PrintWriter;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.LinkedList;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * In-memory implementation of a database of rows and named columns, supporting basic creation and
 * deletion actions, as well as file output in CSV format.
 *
 * @author fhogenboom
 */
public class Database
{

	private String                              name    = "";
	private TreeMap<String, LinkedList<String>> columns = null;

	private static Logger                       _log    = LoggerFactory.getLogger(Database.class);

	/**
	 * Constructor.
	 *
	 * @param name Name of the {@link Database}.
	 */
	public Database(String name)
	{
		this.name = name;
	}

	/**
	 * Initializes the {@link Database} by creating the necessary data structures.
	 */
	public void init()
	{
		_log.info("Initializing storage " + name);
		columns = new TreeMap<String, LinkedList<String>>();
	}

	/**
	 * Flushes the {@link Database} by clearing all the columns.
	 */
	public void flush()
	{
		_log.info("Flushing storage " + name);
		Iterator<String> itHeader = columns.keySet().iterator();
		while (itHeader.hasNext())
		{
			columns.get(itHeader.next()).clear();
		}
	}

	/**
	 * Add column (field) to the {@link Database}.
	 *
	 * @param name Name of field to be added.
	 */
	public void addField(String name)
	{
		columns.put(name, new LinkedList<String>());
	}

	/**
	 * Remove column (field) from the {@link Database}.
	 *
	 * @param name Name of field to be removed.
	 */
	public void removeField(String name)
	{
		columns.remove(name);
	}

	/**
	 * Add row (value) to a column (field) of the {@link Database}.
	 *
	 * @param name Name of field to be updated.
	 * @param value Value to be added.
	 */
	public void addValue(String name, String value)
	{
		columns.get(name).add(value);
	}

	/**
	 * Remove row (value) from a column (field) of the {@link Database}.
	 *
	 * @param name Name of field to be updated.
	 * @param value Value to be removed.
	 */
	public void removeValue(String name, String value)
	{
		columns.get(name).remove(value);
	}

	/**
	 * Save {@link Database} to a CSV formatted file.
	 *
	 * @param path Output folder name (relative paths are allowed).
	 * @param file Output file name (without extension).
	 * @param ext Output file extension (preferably {@code .csv}).
	 */
	public void saveToFile(String path, String file, String ext)
	{
		_log.info("Saving storage " + name + " to file " + path + file + ext);
		try
		{
			// In case path does not exist, create it
			new File(path).mkdirs();
			// Open writer
			PrintWriter pw = new PrintWriter(path + file + ext);
			int max_rows = 0;
			String header = "";
			Iterator<String> itHeader = columns.keySet().iterator();
			// For each column
			while (itHeader.hasNext())
			{
				// Get name
				String column = itHeader.next();
				// Find the number of rows
				int num_rows = columns.get(column).size();
				// If number of rows is larger than the maximum number of rows found
				if (num_rows > max_rows)
				{
					// Update maximum number of rows
					max_rows = num_rows;
				}
				// Store name
				header += column + (itHeader.hasNext() ? ";" : "");
			}
			// Print names
			pw.println(header);
			// For each row
			for (int i = 0; i < max_rows; i++)
			{
				String row = "";
				Iterator<String> itRow = columns.keySet().iterator();
				// For each column
				while (itRow.hasNext())
				{
					String column = itRow.next();
					String value = "";
					// Get value
					try
					{
						value = columns.get(column).remove();
					}
					catch (Exception e)
					{
						_log.warn(e.toString());
					}
					// Add value to current row
					row += value + (itRow.hasNext() ? ";" : "");
				}
				// Print row
				pw.println(row);
			}
			// Close writer
			pw.close();
		}
		catch (Exception e)
		{
			_log.error(e.toString());
		}
	}

}
